## Script for West_Springfield_MA_street_trees.csv
# => Has only common names, so have to translate with Commons Dictionary
library(tidyverse)
library(readxl)
library(taxonomyCleanr)

# Function for case-insensitive joins, nec. for matching CommNames; 
# from https://gist.github.com/jimhester/a060323a05b40c6ada34
# usage: 
# insensitive(left_join)(a, b,  
#    by = list(x = c('letter', 'num'), y = c('letter', 'num2')))
insensitive <- function(fun = inner_join) {
	new_fun <- fun
	body(new_fun) <- substitute({
		by <- dplyr:::common_by(by, x, y)
		tmp_by_x <- paste0("_", by$x, "_")
		tmp_by_y <- paste0("_", by$y, "_")
		for (i in seq_along(by$x)) {
			x[[tmp_by_x[[i]]]] <- tolower(x[[by$x[[i]]]])
			y[[tmp_by_y[[i]]]] <- tolower(y[[by$y[[i]]]])
			y[[by$y[[i]]]] <- NULL
		}
		res <- fun(x, y, list(x = tmp_by_x, y = tmp_by_y))
		res[tmp_by_x] <- list(NULL)
		res
	})
	new_fun
}


# Load data 
#  Change data_path to suit local conditions
data_path <- '../rawInventories/Mass/'
data_file <- 'West_Springfield_MA_street_trees'
data_type <- '.csv'
town_sufx <- '' # To diff among multi files per town, if any
town_name <- 'MA_West_Springfield'
out_path <- paste0('./', town_name)
dir.create(town_name)  # warning if already exists; is fine
# origData <- read_excel(paste0(data_path, data_file, data_type),
# 											 sheet='OGRGeoJSON_4-2014')
origData <- read_csv(paste0(data_path, data_file, data_type)) 
# Can be a heap of warnings; is okay.

# Reduce fields to iTree interests: Sci, Comm, DBH
	# Cambridge: 'species' is common name; 'SpeciesSho' is genus
iTreeData <- origData %>%
	# separate(ident, into=c(NA, NA, 'Species', 'DBH'), sep=':') %>%
	mutate(SciName = NA) %>%
	select(SciName, CommName = Species, DBH_Class = DBH) %>%
	# mutate(CommName = str_replace(CommName, ' *DBH *$', '')) %>%
	# mutate(CommName = str_replace_all(CommName, ' \\"\\w{4}\\"', '')) %>%
	# mutate(CommName = str_replace_all(CommName, '/.*$', ''))
	# mutate(CommName = str_replace_all(CommName, '[`\'".,;:/-]', '')) %>%
	# mutate(CommName = str_replace_all(CommName, '\\s\\s*', ' ')) %>%
	mutate(CommName = trimws(CommName))
iTreeData$SciName <- as.character(iTreeData$SciName)
	# otherwise SciName sits around being boolean

table(iTreeData$CommName, useNA='always') 

## Match common names to SciNames in commons dictionary
#  If commsFile doesn't exist run commDictFull.R
commsFile <- 'commNamesDictionary/commonsDict.csv'
commsDict <- read_csv(commsFile)

comm_matches <- insensitive(left_join)(iTreeData, commsDict,
											by=c('CommName'))
# comm_matches <- left_join(iTreeData, commsDict,
# 											by=c('CommName'))
nonComms <- comm_matches[is.na(comm_matches$SciName.y),]
table(nonComms$CommName, useNA='always')
## Add these to commsDict in commDictFull.R (last step before write_csv)
# Re-join to catch previous nonmatches
comm_matches <- insensitive(left_join)(iTreeData, commsDict,
											by=c('CommName'))

iTreeData <- comm_matches %>% 
	select(SciName = SciName.y, CommName, DBH_Class)

# Get Eco PLANTS codes
spLsFile <- '../EcoSpeciesList_20July2018.csv'
SpecList <- read_csv(spLsFile, skip=1,
								 col_names=c('ID','Code','SName','CName','Genus','Host'))
# eco_matches <- left_join(taxa_map, SpecList,
# 											by=c('taxa_clean' = 'SName'))
eco_matches <- left_join(iTreeData, SpecList,
											by=c('SciName' = 'SName'))
#  Check for records missing codes; should be none
table(eco_matches$CommName[is.na(eco_matches$Code)])

iTreeData <- eco_matches[c(1:3,5)]

# Write csv
outfile = paste0(town_name, town_sufx, '_inventory.csv')
write_csv(iTreeData, paste0(out_path, '/', outfile))
