## Script for Auburn_Trees_All.xlsx
library(tidyverse)
library(readxl)
library(taxonomyCleanr)

# Load data 
#  Change data_path to suit local conditions
data_path <- '../rawInventories/Maine/'
data_file <- 'Auburn_Trees_All'
data_type <- '.xlsx'
town_name <- 'ME_Auburn'
origData <- read_excel(paste0(data_path, data_file, data_type))#,
											 # sheet='Inventory Data')

# Reduce fields to iTree interests: Sci, Comm, DBH
#  Auburn => nothing special?
iTreeData <- origData %>%
# 	separate(`Tree Species`, into=c('SciName', 'CommName'), 
# 									sep=' - ', fill='left') %>%
#   mutate(SciName = str_replace(SciName, " *[\'\"].+$", "")) %>%
	select(SciName = BOTANICAL, CommName = COMMON, DBH)

# Do taxonomyCleanR 
#  Create the taxa map
dir.create(paste0(town_name, '/taxonCleanr'), recursive=TRUE)  # warning if already exists; is fine
out_path <- paste0('./', town_name)
map_path <- paste0('./', town_name, '/taxonCleanr')
taxa_col <- 'SciName'
iTreeData <- as.data.frame(iTreeData)  # TCR rejects tibbles (so far)
taxa_map <- create_taxa_map(path = map_path, x = iTreeData, col = taxa_col)
#  The actual working taxa_map is the file taxa_map.csv; changes get
#  made there, then various functions reload this df from that file.  

#  Get unique taxa and counts
taxa_cts <- count_taxa(x = iTreeData, col = taxa_col, path = map_path)
#  Auburn: n=75

#  Trim excess characters from the taxa list -- 'spp.', e.g.
taxa_map <- trim_taxa(path = map_path)

# # Remove taxa [Included only for reference]
# output <- remove_taxa(path = map_path, input = 'Unsorted biomass')

# # Supported authorities are listed in the column titled resolve_sci_taxa
# view_taxa_authorities()

#  Resolve scientific taxa using ITIS: 3, COL: 1, GBIF: 11
#		Pass 1 of 1 
#taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3,11))
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3))

# # Resolve common (ITIS only)
# # => pretty useless
# outputC <- resolve_comm_taxa(path = map_path, data.sources = 3)

#  See unresolved entries
taxa_map[is.na(taxa_map$taxa_clean), c(1,5)]

#  Correct unresolved entries, as possible
	# Auburn: None!
# repl_list <- list('Fagus grandiflora' = 'Fagus grandifolia',
# 									'Cercidiphyllum japonicum' = 'Cercidiphyllum japonicum')
# map2(names(repl_list), repl_list, function(bad, good) {
# 		replace_taxa(path = map_path, input = bad, output = good)
# 		return()})
# # Should throw a list of NULLs

# # [non-correct move to reload taxa_map from file]
# taxa_map <- trim_taxa(path = map_path)

#  Get changed taxa
changes <- taxa_map[(! taxa_map$taxa_raw %in% taxa_map$taxa_clean)
										& (is.na(taxa_map$taxa_trimmed)), c(1:3,5)]


# Get Eco PLANTS codes
spLsFile <- '../EcoSpeciesList_20July2018.csv'
SpecList <- read_csv(spLsFile, skip=1,
								 col_names=c('ID','Code','SName','CName','Genus','Host'))
eco_matches <- left_join(taxa_map, SpecList,
											by=c('taxa_clean' = 'SName'))

#  Check for records that didn't find codes
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)


	# Auburn: unmatched on Eco list
	#   Some of these are identities, to reset names that get changed
	#   by TCR authorities
repl_list <- list('Acer x freemanii' = 'Acer x freemanii',
									'Carya tomentosa' = 'Carya alba',
									'Gleditsia triacanthos inermis' = 'Gleditsia triacanthos',
									'Rhus typhina' = 'Rhus hirta',
									'Platanus x acerifolia' = 'Platanus x acerifolia')
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})
# Should throw a list of NULLs

# [non-correct move to reload taxa_map from file]
taxa_map <- trim_taxa(path = map_path)

# Make modified taxa_map2 df forcing Eco names into $taxa_clean 
#  => *Does Not Match File*
taxa_map2 <- taxa_map %>% 
	mutate(taxa_clean = if_else(is.na(taxa_replacement), 
															taxa_clean, taxa_replacement))

#  Check again for missing codes (against taxa_map2) -- should be none
eco_matches <- left_join(taxa_map2, SpecList,
											by=c('taxa_clean' = 'SName'))
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)

#  Get changed taxa; write csv of changed names
changes <- taxa_map2[(! taxa_map2$taxa_raw %in% taxa_map2$taxa_clean)
										& (is.na(taxa_map2$taxa_trimmed)), c(1:3,5)]
write_csv(changes, paste0(map_path, '/taxa_changed.csv'))

# => Hackish: overwrite taxa_map.csv with taxa_map2
write_csv(taxa_map2, paste0(map_path, '/taxa_map.csv'))

#  Revise the raw data table (from rewritten taxa_map.csv)
#  and write to file
taxa_output <- revise_taxa(path = map_path, x = iTreeData, col = 'SciName', sep = '\t')

# Reduce to what iTree Eco wants
taxa_output <- taxa_output[1:5]

# Stick on PLANTS codes
taxa_output <- left_join(taxa_output, SpecList,
											by=c('taxa_clean' = 'SName'))[-c(6,8:10)]

table(taxa_output$SciName[is.na(taxa_output$Code)])
table(taxa_output$taxa_clean[is.na(taxa_output$Code)])

# Write csv
outfile = paste0(town_name, '_inventory.csv')
write_csv(taxa_output, paste0(out_path, '/', outfile))

