## Script for 
##  NY_Hastings-on-Hudson_inventoryforarchive.csv
library(tidyverse)
library(readxl)
library(taxonomyCleanr)

# Load data 
#  Change data_path to suit local conditions
data_path <- '../rawInventories/NewYork/'
data_file <- 'NY_Hastings-on-Hudson_inventoryforarchive'
data_type <- '.csv'
town_sufx <- '' # To diff among multi files per town, if any
town_name <- 'NY_Hastings-on-Hudson'
# origData <- read_excel(paste0(data_path, data_file, data_type),
# 											 sheet='Concord_MA_Davey_myTreeKeeper_e') 
origData <- read_csv(paste0(data_path, data_file, data_type)) 
# May produces warnings -- is generally okay

# Reduce fields to iTree interests: Sci, Comm, DBH
iTreeData <- origData %>%
# 	separate(Species, into=c('CommName', 'SciName'),
# 									sep=' \\(', fill='left') %>%
#   mutate(SciName = str_replace(SciName, "\\)", "")) %>%
  # mutate(SciName = str_replace(BOTANICAL, " *[\'\"].+$", "")) %>%
	# mutate(SciName = paste(Genus, Species)) %>%
  mutate(CommName = NA) %>%
	# mutate(SciName = str_to_sentence(SciName)) %>%
	select(SciName = Botanic, 
				 CommName, 
				 DBH = DBH,
				 origCode = SpCode)
# # 1259 Ulmus carpinifolia changed to U. minor
# iTreeData$SciName[str_detect(iTreeData$SciName, 'carpinifolia')] <- 'Ulmus minor'
# table(iTreeData$SciName)

# Do taxonomyCleanR 
#  Create the taxa map
dir.create(paste0(town_name, '/taxonCleanr', town_sufx), recursive=TRUE)  # warning if already exists; is fine
out_path <- paste0('./', town_name)
map_path <- paste0('./', town_name, '/taxonCleanr', town_sufx)
taxa_col <- 'SciName'
iTreeData <- as.data.frame(iTreeData)  # TCR rejects tibbles (so far)
taxa_map <- create_taxa_map(path = map_path, x = iTreeData, col = taxa_col)
#  The actual working taxa_map is the file taxa_map.csv; changes get
#  made there, then various functions reload this df from that file.  

#  Get unique taxa and counts
taxa_cts <- count_taxa(x = iTreeData, col = taxa_col, path = map_path)
#  Concord: n=197

#  Trim excess characters from the taxa list -- 'spp.', e.g.
taxa_map <- trim_taxa(path = map_path)

# # Remove taxa [Included only for reference]
# output <- remove_taxa(path = map_path, input = 'Unsorted biomass')

# # Supported authorities are listed in the column titled resolve_sci_taxa
# view_taxa_authorities()

#  Resolve scientific taxa using ITIS: 3, COL: 1, GBIF: 11
#		Pass 1 of 1 
#taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3,11))
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3))

# # Resolve common (ITIS only)
# # => pretty useless
# outputC <- resolve_comm_taxa(path = map_path, data.sources = 3)

#  See unresolved entries
taxa_map[is.na(taxa_map$taxa_clean), c(1,5)]

#  Correct unresolved entries, as possible
#    Identities are for asserting taxa in Eco list but missed by ITIS
#    Commented entries are for trying to accumulate a list of all these
repl_list <- list(
# These first aren't in ITIS or Eco
	# 'Heptacodium miconioides' = 'Heptacodium miconioides',
	# 'Sciadopitys verticillata' = 'Sciadopitys verticillata',
# Not in ITIS
	# 'Acer cappadocicum' = 'Acer cappadocicum',
	# 'Acer miyabei' = 'Acer miyabei',
	# 'Acer triflorum' = 'Acer triflorum',
	# 'Acer truncatum' = 'Acer truncatum',
	# 'Acer truncatum x platanoides' = 'Acer truncatum',
	# 'Betula jacquemonti' = 'Betula utilis',  
	# 'Cedrus atlantica' = 'Cedrus atlantica',
	# 'Cercidiphyllum japonicum' = 'Cercidiphyllum japonicum', # on web
	# 'Cornus x rutgersensis' = 'Cornus',
	# 'Hamamelis x intermedia' = 'Hamamelis',
	# 'Larix decidua pendula' = 'Larix decidua',
	# 'Magnolia x loebneri' = 'Magnolia',
	# 'Picea orientalis' = 'Picea orientalis',
	# 'Stewartia pseudocamellia' = 'Stewartia pseudocamellia',
	'Prunus sargentii' = 'Prunus sargentii',
	# 'Prunus sargentii columnaris' = 'Prunus sargentii',
	# 'Prunus subhirtella autumnalis' = 'Prunus subhirtella',
	# 'Salix matsudana tortuosa' = 'Salix matsudana',
	# 'Thuja occidentalis filiformis' = 'Thuja occidentalis',
	# 'Zelkova sinica' = 'Zelkova sinica')
# Typos, etc.
	# 'Cercis canadensis alba' = 'Cercis canadensis',
	# 'Chamaecyparis: spp.' = 'Chamaecyparis',
	# 'Crataegus crusgalli' = 'Crataegus crus-galli',
	'Crataegus crusgalli' = 'Crataegus crus-galli'#,
	# 'Crateagus crusgalli inermis' = 'Crataegus crus-galli',
	# 'Crataegus superba' = 'Crataegus laevigata',
	# 'Fagus sylvatica Tricolor' = 'Fagus sylvatica',
	# 'Fagus sylvatica f. purpurea' = 'Fagus sylvatica'#,
	# 'Gingko biloba' = 'Ginkgo biloba',
	# 'Ginko Biloba' = 'Ginkgo biloba',
	# 'Ginko; Biloba' = 'Ginkgo biloba',
	# 'Ginko biloba princeton sentry' = 'Ginkgo biloba',
	# 'Gymnocladus dioica' = 'Gymnocladus dioicus'#,
	# 'unknown Cladrastis' = 'Cladrastis',
	# 'Prunus x incam' = 'Prunus'
	# 'Quercus rubrum' = 'Quercus rubra',
	# 'X Cupressocyparis leylandii' = 'Cupressocyparis leylandii'
	# 'Elm' = 'Ulmus'
)
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})
# Should throw a list of NULLs

# # [non-correct move to reload taxa_map from file]
taxa_map <- trim_taxa(path = map_path)

#  Get changed taxa
changes <- taxa_map[(! taxa_map$taxa_raw %in% taxa_map$taxa_clean)
										& (is.na(taxa_map$taxa_trimmed)), c(1:3,5)]


# Get Eco PLANTS codes
spLsFile <- '../EcoSpeciesList_20July2018.csv'
SpecList <- read_csv(spLsFile, skip=1,
								 col_names=c('ID','Code','SName','CName','Genus','Host'))
eco_matches <- left_join(taxa_map, SpecList,
											by=c('taxa_clean' = 'SName'))

#  Check for records that didn't find codes
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)


	# Unmatched on Eco list
	#   Some of these are identities, to reset names that get changed
	#   by TCR authorities
repl_list <- list(
	# 'Abies balsamea var phanerolepis' = 'Abies x phanerolepis',
	# 'Acer freemanii' = 'Acer x freemanii',
	'Acer x freemanii' = 'Acer x freemanii',
	# 'Acer tataricum ginnala' = 'Acer tataricum',
	# 'Aesculus x carnea' = 'Aesculus x carnea',
	# 'Alnus rugosa' = 'Alnus incana',
	# 'Amelanchier grandifolia' = 'Amelanchier x grandiflora',
	# 'Amelanchier x grandiflora' = 'Amelanchier x grandiflora',
	# 'Carya tomentosa' = 'Carya alba',
	# 'Cladrastis lutea' = 'Cladrastis kentukea',
	# 'Gleditsia triacanthos inermis' = 'Gleditsia triacanthos',
	# 'Laburnum x watereri' = 'Laburnum x watereri',
	# 'Laburnum_x_watereri' = 'Laburnum x watereri',
	# 'Magnolia x soulangiana' = 'Magnolia x soulangiana',
	# 'Magnolia soulangiana' = 'Magnolia x soulangiana',
	# 'Picea pungens glauca' = 'Picea pungens'
	# 'Plantanus x acerifolia' = 'Platanus x acerifolia',
	# 'Plantanus acerifolia' = 'Platanus x acerifolia'#,
	# 'Platanus acerifolia' = 'Platanus x acerifolia',
	'Platanus x acerfolia' = 'Platanus x acerifolia',
	'Platanus x acerifolia' = 'Platanus x acerifolia',
	# 'Quercus nuttallii' = 'Quercus texana',
	# 'Rhus typhina' = 'Rhus hirta',
	# 'Styphnolobium japonicum' = 'Sophora japonica'
	'Styphnolobium japonica' = 'Sophora japonica'
	# 'Tilia x europaea' = 'Tilia x europaea',
	# 'Ulmus x hollandica' = 'Ulmus x hollandica'
)
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})
# Should throw a list of NULLs

# [non-correct move to reload taxa_map from file]
taxa_map <- trim_taxa(path = map_path)

# Make modified taxa_map2 df forcing Eco names into $taxa_clean 
#  => *Does Not Match File*
taxa_map2 <- taxa_map %>% 
	mutate(taxa_clean = if_else(is.na(taxa_replacement), 
															taxa_clean, taxa_replacement))

#  Check again for missing codes (against taxa_map2) -- should be none
eco_matches <- left_join(taxa_map2, SpecList,
											by=c('taxa_clean' = 'SName'))
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)

#  Get changed taxa; write csv of changed names
changes <- taxa_map2[(! taxa_map2$taxa_raw %in% taxa_map2$taxa_clean)
										& (is.na(taxa_map2$taxa_trimmed)), c(1:3,5)]
write_csv(changes, paste0(map_path, '/taxa_changed.csv'))

# => Hackish: overwrite taxa_map.csv with taxa_map2
write_csv(taxa_map2, paste0(map_path, '/taxa_map.csv'))

#  Revise the raw data table (from rewritten taxa_map.csv)
#  and write to file
taxa_output <- revise_taxa(path = map_path, x = iTreeData, 
													 col = 'SciName', sep = '\t')

# Reduce to what iTree Eco wants
taxa_output <- taxa_output[1:5]

# Stick on PLANTS codes
taxa_output <- left_join(taxa_output, SpecList,
											by=c('taxa_clean' = 'SName'))[-c(6,8:10)]

table(taxa_output$taxa_clean, useNA='always')
table(taxa_output$SciName[is.na(taxa_output$Code)])
table(taxa_output$taxa_clean[is.na(taxa_output$Code)])

# Write csv
outfile = paste0(town_name, town_sufx, '_inventory.csv')
write_csv(taxa_output, paste0(out_path, '/', outfile))

