## Script for VT_Urban_Tree_Inventory.xlsx
library(tidyverse)
library(readxl)
library(taxonomyCleanr)

## First try, straight out of .xlsx
# # Load data 
# data_path <- '../InvsForClarke/'
# data_file <- 'VT_Urban_Tree_Inventory'
# data_type <- '.xlsx'
# origData <- read_excel(paste(data_path, data_file, data_type, sep=''),
# 											 sheet='VT_Urban_Tree_Inventory')

## Second try, using "Species comment" files
# Load data
VTpath <- 'VTinventories/'
VTall <- list.files(path=VTpath, pattern="*.csv", full.names=TRUE) %>% 
    map_df(~read_csv(., col_types = cols(.default = "c")))

# Replace bogus 'XXX OTHER' codes where Species_comment is useful
VTall$SpeciesID[str_detect(VTall$Species_Comments, 'Black cherry')] <- 'PRSE1'
VTall$SpeciesID[str_detect(VTall$Species_Comments, 'Cedar')] <- 'CE7'
VTall$SpeciesID[str_detect(VTall$Species_Comments, 'Hackberry')] <- 'CE2'
VTall$SpeciesID[str_detect(VTall$Species_Comments, 'Hemlock')] <- 'TSCA'
VTall$SpeciesID[str_detect(VTall$Species_Comments, 'Magnolia')] <- 'MA1'
VTall$SpeciesID[str_detect(VTall$Species_Comments, 'Ornamental')] <- 'PR'
VTall$SpeciesID[str_detect(VTall$Species_Comments, 'Peach')] <- 'PRPE2'
VTall$SpeciesID[str_detect(VTall$Species_Comments, 'Sycamore')] <- 'PLOC'
VTall$SpeciesID[str_detect(VTall$Species_Comments, 'Upright')] <- 'TA'

# Reduce fields to iTree interests: Sci, Comm, DBH
# => VT only lists PLANTS codes
# iTreeData <- origData %>%
# 	mutate(SciName = NA, CommName = NA) %>%
# 	select(Town = TownName, SciName, CommName, DBH = Diameter, Code = SpeciesID)

iTreeData2 <- VTall %>%
	mutate(SciName = NA, CommName = NA) %>%
	select(Town = TownName, SciName, CommName, DBH = Diameter, Code = SpeciesID)

table(iTreeData2$Code)
table(iTreeData2$Code[str_detect(iTreeData2$Code, 'OTHER')])

# # Remove entries with no Town
# iTreeData <- iTreeData[! is.na(iTreeData$Town),]

# Do taxonomyCleanR 
# => No tax cleaning -- VT only has the codes


# Write per-town csvs
statePref = 'VT_'
# dataList <- split(iTreeData, iTreeData$Town) 
# mapply(write_csv, dataList, path = paste0(statePref, names(dataList), '_', 'inventory.csv'))

dataList2 <- split(iTreeData2, iTreeData2$Town) 
# dataList3 <- split(iTreeData3, iTreeData3$Town) 
mapply(write_csv, dataList2, path = paste0(statePref, names(dataList2), '_', 'inventory.csv'))
