## Script for 
##  VT_Middlebury_inventoryforarchive.csv
library(tidyverse)
library(readxl)
library(taxonomyCleanr)

# Load data 
#  Change data_path to suit local conditions
data_path <- '../rawInventories/Vermont/'
data_file <- 'VT_Middlebury_inventoryforarchive'
data_type <- '.csv'
town_sufx <- '' # To diff among multi files per town, if any
town_name <- 'VT_Middlebury'
# origData <- read_excel(paste0(data_path, data_file, data_type),
# 											 sheet='Concord_MA_Davey_myTreeKeeper_e') 
origData <- read_csv(paste0(data_path, data_file, data_type)) 
# May produces warnings -- is generally okay

# Reduce fields to iTree interests: Sci, Comm, DBH
iTreeData <- origData %>%
	# separate(Species, into=c('CommName', 'SciName'),
	# 								sep=' \\(', fill='left') %>%
  # mutate(SciName = str_replace(SciName, "\\)", "")) %>%
  mutate(SciName = str_replace(LatinName, " *[\'\"].+$", "")) %>%
  # mutate(SciName = str_replace_all(LATINNAME, "[\'\"]", "")) %>%
	# mutate(SciName = paste(Genus, Species)) %>%
  # mutate(CommName = NA) %>%
	mutate(SciName = str_to_sentence(SciName)) %>%
	select(SciName, 
				 CommName = `Common Name`, 
					 DBH)

# Do taxonomyCleanR 
#  Create the taxa map
dir.create(paste0(town_name, '/taxonCleanr', town_sufx), recursive=TRUE)  # warning if already exists; is fine
out_path <- paste0('./', town_name)
map_path <- paste0('./', town_name, '/taxonCleanr', town_sufx)
taxa_col <- 'SciName'

iTreeData <- as.data.frame(iTreeData)  # TCR rejects tibbles (so far)
taxa_map <- create_taxa_map(path = map_path, x = iTreeData, col = taxa_col)
#  The actual working taxa_map is the file taxa_map.csv; changes get
#  made there, then various functions reload this df from that file.  

#  Get unique taxa and counts
taxa_cts <- count_taxa(x = iTreeData, col = taxa_col, path = map_path)

#  Trim excess characters from the taxa list -- 'spp.', e.g.
taxa_map <- trim_taxa(path = map_path)

# # Remove taxa [Included only for reference]
# output <- remove_taxa(path = map_path, input = 'Unsorted biomass')

# # Supported authorities are listed in the column titled resolve_sci_taxa
# view_taxa_authorities()

#  Resolve scientific taxa using ITIS: 3, COL: 1, GBIF: 11
#		Pass 1 of 1 
#taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3,11))
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3))

#  See unresolved entries
arrange(taxa_map[is.na(taxa_map$taxa_clean), c(1,5)], taxa_raw)

#  Correct unresolved entries, as possible
#    Identities are for asserting taxa in Eco list but missed by ITIS
#    Commented entries are for trying to accumulate a list of all these
repl_list <- list(
# These first aren't in ITIS or Eco
	'Heptacodium miconioides' = 'Heptacodium miconioides',
	# 'Sciadopitys verticillata' = 'Sciadopitys verticillata',
# Not in ITIS
	# 'Abies nordmanniana' = 'Abies nordmanniana',
	# 'Acer cappadocicum' = 'Acer cappadocicum',
	# 'Acer heldreichii' = 'Acer heldreichii',
	'Acer miyabei' = 'Acer miyabei',
	# 'Acer mandshuricum' = 'Acer mandshuricum',
	# 'Acer monspessulanum' = 'Acer monspessulanum',
	# 'Acer platanoides columnare' = 'Acer platanoides',
	'Acer pseudosieboldianum' = 'Acer pseudosieboldianum',
	# 'Acer rufinerve' = 'Acer rufinerve',
	'Acer triflorum' = 'Acer triflorum',
	# 'Acer truncatum' = 'Acer truncatum',
	# 'Acer x truncatum' = 'Acer truncatum',
	# 'Acer truncatum x platanoides' = 'Acer truncatum',
	'Acer tschonoskii' = 'Acer',
	# 'Benthamidia' = 'Cornus',
	# 'Benthamidia florida' = 'Cornus florida',
	# 'Benthamidia japonica' = 'Cornus',
	# 'Betula jacquemonti' = 'Betula utilis',  
	# 'Betula pendula gracilis' = 'Betula pendula gracilis',
	# 'Betula utilis var. Jacquemontii' = 'Betula utilis',  
	# 'Carpinus japonica' = 'Carpinus japonica',
	# 'Catalpa erubescens' = 'Catalpa erubescens',
	# 'Cedrus atlantica' = 'Cedrus atlantica',
	'Cercidiphyllum japonicum' = 'Cercidiphyllum japonicum', # on web
	# 'Cercis chinensis' = 'Cercis chinensis',
	# 'Cercis reniformis oklahoma' = 'Cercis reniformis',
	# 'Cornus x rutgersensis' = 'Cornus',
	# 'Cupressus x leylandii' = 'Cupressocyparis leylandii',
	# 'Davidia involucrata' = 'Davidia involucrata',
	# 'Genista lydia' = 'Genista',
	'Eleagnus angustifolia' = 'Eleagnus angustifolia',
	# 'Hamamelis mollis' = 'Hamamelis mollis',
	# 'Hamamelis x intermedia' = 'Hamamelis',
	# 'Hypericum erectum' = 'Hypericum',
	# 'Ilex x aquipernyi' = 'Ilex',
	# 'Larix decidua pendula' = 'Larix decidua',
	# 'Magnolia x loebneri' = 'Magnolia',
	# 'Malus x moerlandsii' = 'Malus',
	# 'Picea orientalis' = 'Picea orientalis',
	# 'Prunus sargentii' = 'Prunus sargentii',
	# 'Prunus sargentii columnaris' = 'Prunus sargentii',
	# 'Prunus subhirtella autumnalis' = 'Prunus subhirtella',
	# 'Pyrus fauriei' = 'Pyrus fauriei',
	# 'Quercus x warei' = 'Quercus',
	# 'Salix matsudana tortuosa' = 'Salix matsudana',
	# 'Spiraea decumbens' = 'Spiraea',
	# 'Stewartia pseudocamellia' = 'Stewartia pseudocamellia',
	# 'Styrax obassia' = 'Styrax obassia',
	# 'Thuja occidentalis filiformis' = 'Thuja occidentalis',
	# 'Tsuga diversifolia' = 'Tsuga',
	# 'Ulmus propinqua' = 'Ulmus davidiana',
	# 'Xanthocyparis nootkatensis' = 'Chamaecyparis nootkatensis',
	# 'Zelkova sinica' = 'Zelkova sinica'),
# Typos, etc.
	# 'Acer palmatum matsumurae group' = 'Acer palmatum',
	# 'Acer trunctatum' = 'Acer truncatum',
	# 'Apricot/plum hybrid' = 'Prunus',
	# 'Buddleia davidii' = 'Buddleja davidii',
	# 'Carpinus betulus fastigiata' = 'Carpinus betulus',
	# 'Carpinus betulus var. fastigiata' = 'Carpinus betulus',
	# 'Carya illinoensis' = 'Carya illinoinensis',
	# 'Cedrus libani ssp. atlantica' = 'Cedrus libani',
	# 'Cedrus libani ssp. libani' = 'Cedrus libani',
	# 'Cercidiphylum japonicum' = 'Cercidiphyllum japonicum', # on web
	# 'Cercis canadensis alba' = 'Cercis canadensis',
	# 'Chamaecyparis: spp.' = 'Chamaecyparis',
	# 'Crab apple' = 'Malus',
	# 'Crataegus crusgalli' = 'Crataegus crus-galli',
	# 'Crataegus crusgalli' = 'Crataegus crus-galli',
	# 'Crataegus crusgalli var.' = 'Crataegus crus-galli',
	# 'Crateagus crusgalli inermis' = 'Crataegus crus-galli',
	# 'Crataegus superba' = 'Crataegus laevigata',
	'Eleagnus angustifolia' = 'Elaeagnus angustifolia',
	# 'Elm' = 'Ulmus',
	# 'European hornbeam' = 'Carpinus betulus'
	# 'Fagus sylvatica dawyckii' = 'Fagus sylvatica',
	# 'Fagus sylvatica pendula' = 'Fagus sylvatica',
	# 'Fagus sylvatica tricolor' = 'Fagus sylvatica',
	# 'Fagus sylvatica Tricolor' = 'Fagus sylvatica',
	# 'Fagus sylvatica f. purpurea' = 'Fagus sylvatica'#,
	# 'Fagus sylvatica purpurea group' = 'Fagus sylvatica',
	# 'Fraxinus pennsylvania' = 'Fraxinus pennsylvanica',
	# 'Gingko biloba' = 'Ginkgo biloba',
	# 'Ginko Biloba' = 'Ginkgo biloba',
	# 'Ginko; Biloba' = 'Ginkgo biloba',
	# 'Ginko biloba princeton sentry' = 'Ginkgo biloba',
	'Gymnocladus dioica' = 'Gymnocladus dioicus',
	# 'Ilex x meserveae' = 'Ilex',
	# 'Maagolia x soulangiana' = 'Magnolia x soulangiana',
	'Malus x hybrida' = 'Malus',
	# 'Myrica pennsylvanica' = 'Myrica pensylvanica',
	# 'Oxydendron arboreum' = 'Oxydendrum arboreum',
	# 'Picea glauca conica' = 'Picea glauca',
	# 'Picea omorica' = 'Picea omorika',
	# 'Prunus cistena' = 'Prunus x cistena',
	# 'Prunus pennsylvanica' = 'Prunus pensylvanica',
	# 'Prunus serrulata pendula' = 'Prunus serrulata',
	# 'Prunus serrulata ssp. pubescens' = 'Prunus serrulata',
	# 'Prunus subhirtella pendula' = 'Prunus subhirtella',
	'Prunus x hybrida' = 'Prunus'
	# 'Prunus x incam' = 'Prunus'
	# 'Quercus rubrum' = 'Quercus rubra',
	# 'Quercus x hybrida' = 'Quercus',
	# 'Qurecus rubra' = 'Quercus rubra',
	# 'Salix arenaria' = 'Salix repens',
	# 'Sciadopity verticillata' = 'Sciadopitys verticillata',
	# 'Sequoiadendron gigantea' = 'Sequoiadendron giganteum',
	# 'Spiraea x hybrida' = 'Spiraea',
	# 'Stewartia pseudocamillia' = 'Stewartia pseudocamellia'
	# 'Tilia x mongolica' = 'Tilia mongolica'
	# 'Ulmus glabra camperdownii' = 'Ulmus glabra'
	# 'Ulmus thomasi' = 'Ulmus thomasii'
	# 'X cupressocyparis leylandii' = 'Cupressocyparis leylandii'
)
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})
# Should throw a list of NULLs

# # [non-correct move to reload taxa_map from file]
taxa_map <- trim_taxa(path = map_path)

### Do hack thing here -- first pass
# Make modified taxa_map2 df forcing ITIS misses into $taxa_clean 
#  => *Does Not Match File*
taxa_map2 <- taxa_map %>% 
	mutate(taxa_clean = ifelse(is.na(taxa_replacement), 
															taxa_clean, taxa_replacement))

# => Hackish: overwrite taxa_map.csv with taxa_map2
write_csv(taxa_map2, paste0(map_path, '/taxa_map.csv'))


#  Get changed taxa
changes <- taxa_map2[(! taxa_map2$taxa_raw %in% taxa_map2$taxa_clean)
										& (is.na(taxa_map2$taxa_trimmed)), c(1:3,5)]

# Get Eco PLANTS codes
spLsFile <- '../EcoSpeciesList_20July2018.csv'
SpecList <- read_csv(spLsFile, skip=1,
								 col_names=c('ID','Code','SName','CName','Genus','Host'))
eco_matches <- left_join(taxa_map2, SpecList,
											by=c('taxa_clean' = 'SName'))

#  Check for records that didn't find codes
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)

	# Unmatched on Eco list
	#   Some of these are identities, to reset names that get changed
	#   by TCR authorities
repl_list <- list(
	# 'Abies balsamea var phanerolepis' = 'Abies x phanerolepis',
	# 'Acer freemanii' = 'Acer x freemanii',
	'Acer x freemanii' = 'Acer x freemanii',
	# 'Acer platanoides schwedleri' = 'Acer platanoides',
	# 'Acer tataricum ginnala' = 'Acer tataricum',
	# 'Acer tataricum ssp. ginnala' = ' Acer tataricum',
	# 'Aesculus octandra' = 'Aesculus flava',
	# 'Aesculus x arnoldiana' = 'Aesculus',
	# 'Aesculus x carnea' = 'Aesculus x carnea',
	# 'Alnus rugosa' = 'Alnus incana',
	# 'Amelanchier grandifolia' = 'Amelanchier x grandiflora',
	# 'Amelanchier x grandiflora' = 'Amelanchier x grandiflora',
	# 'Amelanchier x grandiflora [autumn brilliance]' = 'Amelanchier x grandiflora',
	# 'Betula szechuanica' = 'Betula platyphylla',
	# 'Buxus microphyla' = 'Buxus microphylla koreana',
	# 'Carya tomentosa' = 'Carya alba',
	# 'Cerciciphylum japonicum' = 'Cercidiphyllum japonicum',
	# 'Cladrastis lutea' = 'Cladrastis kentukea',
	# 'Crataegus crusgalli var. Inermis' = 'Crataegus crus-galli',
	# 'Crataegus oxyacantha' = 'Crataegus',
	# 'Enkianthus campanulatus' = 'Enkianthus campanulatus',
	# 'Forsythia x intermedia' = 'Forsythia x intermedia', # Web returns 'Forsythia X intermedia'
	# 'Gleditsia triacanthos forma inermis' = 'Gleditsia triacanthos',
	# 'Gleditsia triacanthos inermis' = 'Gleditsia triacanthos',
	# 'Ilex pedunculosa' = 'Ilex',
	# 'Laburnum watereri' = 'Laburnum x watereri',
	# 'Laburnum x watereri' = 'Laburnum x watereri',
	# 'Laburnum_x_watereri' = 'Laburnum x watereri',
	# 'Magnolia soulangiana' = 'Magnolia x soulangiana',
	# 'Magnolia x soulangeana' = 'Magnolia x soulangiana',
	# 'Magnolia x soulangiana' = 'Magnolia x soulangiana',
	# 'Myrica heterophylla' = 'Myrica',
	# 'Myrica pennsylvanica' = 'Myrica',
	# 'Paxistima canbyii' = 'Paxistima', # also should be 'canbyi'
	# 'Picea omorica' = 'Picea omorika',
	# 'Picea pungens glauca' = 'Picea pungens',
	# 'Picea pungens glauca group' = 'Picea pungens',
	# 'Plantanus x acerifolia' = 'Platanus x acerifolia',
	# 'Plantanus acerifolia' = 'Platanus x acerifolia'#,
	# 'Platanus acerifolia' = 'Platanus x acerifolia',
	# 'Platanus acerifoli' = 'Platanus x acerifolia',
	# 'Platanus x acerfolia' = 'Platanus x acerifolia'
	'Platanus x acerifolia' = 'Platanus x acerifolia',
	# 'Potentilla fruiticosa' = 'Potentilla', # Eco has typo
	# 'Quercus nuttallii' = 'Quercus texana',
	# 'Quercus robur forma fastigiata' = 'Quercus robur',
	# 'Quercus robur fastigiata' = 'Quercus robur',
	# 'Rhamnus frangula' = 'Frangula alnus'
	# 'Rhus typhina' = 'Rhus hirta'
	# 'Rosa hugonis' = 'Rosa',
	# 'Salix arenaria' = 'Salix',
	# 'Salix x cottetii' = 'Salix',
	# 'Salix x grahamii' = 'Salix',
	# 'Salix x sepulcralis simonkai' = 'Salix x sepulcralis Simonkai'
	# 'Spirea x bumalda' = 'Spiraea x bumalda',
	# 'Styphnolobium japonicum' = 'Sophora japonica',
	# 'Styphnoloboum japonicum' = 'Sophora japonica',
	# 'Styphnolobium japonica' = 'Sophora japonica',
	# 'Swida alternifolia' = 'Cornus alternifolia',
	# 'Syringa meyeri palibin' = 'Syringa',
	'Syringa pekinensis' = 'Syringa reticulata',
	# 'Syringa persica' = 'Syringa x persica',
	# 'Thuja orientalis' = 'Platycladus orientalis',
	# 'Tilia x europa' = 'Tilia x europaea',
	# 'Tilia x europaea' = 'Tilia x europaea',
	# 'Tilia x flavescens' = 'Tilia',
	# 'Ulmus capinifolia' = 'Ulmus'
	# 'Ulmus carpinifolia' = 'Ulmus',
	'Ulmus davidiana var. japonica' = 'Ulmus davidiana'
	# 'Ulmus x hollandica' = 'Ulmus x hollandica'
	# 'Viburnum sargentii' = 'Viburnum'
	# 'Viburnum trilobum' = 'Viburnum'
)
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})
# Should throw a list of NULLs

# [non-correct move to reload taxa_map from file]
taxa_map <- trim_taxa(path = map_path)

# Make modified taxa_map2 df forcing Eco names into $taxa_clean 
#  => *Does Not Match File*
taxa_map2 <- taxa_map %>% 
	mutate(taxa_clean = if_else(is.na(taxa_replacement), 
															taxa_clean, taxa_replacement))

#  Check again for missing codes (against taxa_map2) -- should be none
eco_matches <- left_join(taxa_map2, SpecList,
											by=c('taxa_clean' = 'SName'))
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)

#  Get changed taxa; write csv of changed names
changes <- taxa_map2[(! taxa_map2$taxa_raw %in% taxa_map2$taxa_clean)
										& (is.na(taxa_map2$taxa_trimmed)), c(1:3,5)]
write_csv(changes, paste0(map_path, '/taxa_changed.csv'))

# => Hackish: overwrite taxa_map.csv with taxa_map2
write_csv(taxa_map2, paste0(map_path, '/taxa_map.csv'))

#  Revise the raw data table (from rewritten taxa_map.csv)
#  and write to file
taxa_output <- revise_taxa(path = map_path, x = iTreeData, 
													 col = 'SciName', sep = '\t')

# Reduce to what iTree Eco wants
taxa_output <- taxa_output[1:5]

# Stick on PLANTS codes
taxa_output <- left_join(taxa_output, SpecList,
											by=c('taxa_clean' = 'SName'))[-c(6,8:10)]

table(taxa_output$taxa_clean, useNA='always')
table(taxa_output$SciName[is.na(taxa_output$Code)])
table(taxa_output$taxa_clean[is.na(taxa_output$Code)])

# Write csv
outfile = paste0(town_name, town_sufx, '_inventory.csv')
write_csv(taxa_output, paste0(out_path, '/', outfile))

