## Organizing dictionary of common names

library(tidyverse)
library(readxl)
library(taxonomyCleanr)

# First pile up a df of all trees
	# Presumes working dir contains town output dirs
	# wipe out quotes and other punctuation (only way to standardize)
	# make lower case

# First-pass sources
# sources <- c('CT_Inventories', 
# 						 'NA_Webster', 'MA_Turners_Falls',
# 						 'NY_Syracuse', 'MA_Springfield')

# All-output sources
sources <- list.files(pattern='^[A-Z]{2}_', recursive=FALSE) 
# Feb. 28, 2020: this gets 38 files
# March 12: 54 files

themOnes <- do.call(bind_rows, lapply(
	# list.files(sources, '.csv', full.names=TRUE), read_csv)) %>%
	list.files(sources, '.csv', full.names=TRUE), function(i) {
		read_csv(i, col_types=cols_only(SciName='c', CommName='c'))})) %>%
	mutate(CommName = str_replace_all(CommName, '[`\'".,;:()/]', '')) %>%
	mutate(commname = tolower(CommName)) %>%
	mutate(SciName = str_replace_all(SciName, '[`\'".,;:]', '')) %>%
	mutate(SciName = str_to_sentence(SciName)) %>%
	mutate(SciName = str_replace_all(SciName, 'species', ''))
# with the 38 files in sources, n=472410
# with 54 files, n=615934

#themOnes[str_detect(themOnes$CommName, '[Ss]pp') & !is.na(themOnes$CommName),]
length(unique(themOnes$SciName))  # n=951 -- includes typos & detritus
length(unique(themOnes$CommName)) # n=2823

# Reduce megalist to all unique pairings of CommName and Sciname
commsDict <- arrange(unique(themOnes[c(1,3)]), SciName) %>%
	transmute(SciName = SciName, CommName = commname) %>%
	.[complete.cases(.),] # n=2356

# check some examples
cherries <- commsDict[str_detect(commsDict$CommName, 'cherry'),] # n=137
commsDict[str_detect(commsDict$SciName, 'Sorbus'),]  # n=28


## Do taxonomyCleanR 
	# The resolve() here takes a looong time.
dir.create(paste0('commNamesDictionary', '/taxonCleanr'), recursive=TRUE)
out_path <- paste0('./', 'commNamesDictionary')
map_path <- paste0('./', 'commNamesDictionary', '/taxonCleanr')
taxa_col <- 'SciName'
commsDict <- as.data.frame(commsDict)  # TCR rejects tibbles (so far)
taxa_map <- create_taxa_map(path = map_path, x = commsDict, col = taxa_col)
# taxa_map n=936
taxa_cts <- count_taxa(x = commsDict, col = taxa_col, path = map_path)
taxa_map <- trim_taxa(path = map_path)
taxa_cts <- count_taxa(x = commsDict, col = taxa_col, path = map_path)
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3))
#  See unresolved entries
taxa_map[is.na(taxa_map$taxa_clean), c(1,5)] -> unresolveds

#  Correct unresolved entries, as possible
#    Identities are for asserting taxa in Eco list but missed by ITIS
#    Commented entries are for trying to accumulate a list of all these
repl_list <- list(
# These first aren't in ITIS or Eco
	'Heptacodium miconioides' = 'Heptacodium miconioides',
	'Sciadopitys verticillata' = 'Sciadopitys verticillata',
# Not in ITIS
	'Abies koreana' = 'Abies koreana',
	'Abies nordmanniana' = 'Abies nordmanniana',
	'Acer cappadocicum' = 'Acer cappadocicum',
	'Acer heldreichii' = 'Acer heldreichii',
	'Acer mandshuricum' = 'Acer mandshuricum',
	'Acer miyabei' = 'Acer miyabei',
	'Acer monspessulanum' = 'Acer monspessulanum',
	'Acer palmatum matsumurae group' = 'Acer palmatum',
	# 'Acer platanoides columnare' = 'Acer platanoides',
	'Acer pseudosieboldianum' = 'Acer pseudosieboldianum',
	'Acer rubrum columnare' = 'Acer rubrum',
	'Acer rufinerve' = 'Acer rufinerve',
	'Acer triflorum' = 'Acer triflorum',
	'Acer truncatum' = 'Acer truncatum',
	'Acer truncatum x platanoides' = 'Acer truncatum',
	'Acer tschonoskii' = 'Acer',
	'Acer x truncatum' = 'Acer truncatum',
	'Benthamidia' = 'Cornus',
	'Benthamidia florida' = 'Cornus florida',
	'Benthamidia japonica' = 'Cornus kousa',
	'Betula jacquemonti' = 'Betula utilis',
	'Betula utilis var jacquemontii' = 'Betula utilis',
	# 'Betula pendula gracilis' = 'Betula pendula gracilis',
	'Carpinus japonica' = 'Carpinus japonica',
	'Catalpa erubescens' = 'Catalpa erubescens',
	'Cedrus atlantica' = 'Cedrus atlantica',
	'Cedrus libani ssp atlantica' = 'Cedrus libani',
	'Cedrus libani ssp libani' = 'Cedrus libani',
	'Cercidiphyllum japonicum' = 'Cercidiphyllum japonicum', # on web
	'Cercis chinensis' = 'Cercis chinensis',
	'Cercis reniformis' = 'Cercis reniformis',
	'Cercis reniformis oklahoma' = 'Cercis reniformis',
	'Cornus x rutgersensis' = 'Cornus',
	'Cupressus x leylandii' = 'Cupressus',
	'Davidia involucrata' = 'Davidia involucrata',
	'Genista lydia' = 'Genista',
	'Hamamelis mollis' = 'Hamamelis mollis',
	'Hamamelis x intermedia' = 'Hamamelis',
	'Hypericum erectum' = 'Hypericum',
	'Larix decidua pendula' = 'Larix decidua',
	'Magnolia x loebneri' = 'Magnolia',
	'Picea orientalis' = 'Picea orientalis',
	'Prunus pissardii' = 'Prunus pissardii',
	'Prunus sargentii' = 'Prunus sargentii',
	'Prunus sargentii columnaris' = 'Prunus sargentii',
	'Prunus subhirtella autumnalis' = 'Prunus subhirtella',
	'Pyrus fauriei' = 'Pyrus fauriei',
	'Quercus x warei' = 'Quercus',
	'Salix arenaria' = 'Salix repens',
	# 'Salix matsudana tortuosa' = 'Salix matsudana',
	'Spiraea decumbens' = 'Spiraea',
	'Stewartia pseudocamellia' = 'Stewartia pseudocamellia',
	'Styrax obassia' = 'Styrax obassia',
	# 'Thuja occidentalis filiformis' = 'Thuja occidentalis',
	'Tilia mongolica' = 'Tilia mongolica',
	'Tsuga diversifolia' = 'Tsuga',
	'Ulmus glabra camperdownii' = 'Ulmus glabra',
	'Ulmus propinqua' = 'Ulmus davidiana',
	'Xanthocyparis nootkatensis' = 'Chamaecyparis nootkatensis',
	'Zelkova sinica' = 'Zelkova sinica',
# Typos, etc.
	# 'Acer trunctatum' = 'Acer truncatum',
	'Apricot/plum hybrid' = 'Prunus',
	'Arbor (thuja occidentalis' = 'Thuja occidentalis',
	'Buddleia davidii' = 'Buddleja davidii',
	'Carpinus betulus fastigiata' = 'Carpinus betulus',
	'Carpinus betulus var fastigiata' = 'Carpinus betulus',
	'Carya illinoensis' = 'Carya illinoinensis',
	'Cerciciphylum japonicum' = 'Cercidiphyllum japonicum',
	'Cercidiphylum japonicum' = 'Cercidiphyllum japonicum',
	'Cercis canadensis alba' = 'Cercis canadensis',
	# 'Chamaecyparis: spp.' = 'Chamaecyparis',
	# 'Crab apple' = 'Malus',
	'Crataegus crusgalli' = 'Crataegus crus-galli',
	'Crataegus crusgalli var' = 'Crataegus crus-galli',
	'Crataegus crusgalli var inermis' = 'Crataegus crus-galli',
	'Crataegus superba' = 'Crataegus laevigata',
	'Crateagus crusgalli inermis' = 'Crataegus crus-galli',
	'Eleagnus angustifolia' = 'Elaeagnus angustifolia',
	'Elm' = 'Ulmus',
	'European hornbeam' = 'Carpinus betulus',
	'Evodia danielli' = 'Tetradium daniellii',
	'Fagus grandiflora' = 'Fagus grandifolia',
	'Fagus silvatica' = 'Fagus sylvatica',
	'Fagus sylvatica dawyckii' = 'Fagus sylvatica',
	'Fagus sylvatica pendula' = 'Fagus sylvatica',
	'Fagus sylvatica purpurea group' = 'Fagus sylvatica',
	'Fagus sylvatica tricolor' = 'Fagus sylvatica',
	# 'Fagus sylvatica f. purpurea' = 'Fagus sylvatica',
	'Fraxinus pennsylvania' = 'Fraxinus pennsylvanica',
	'Gingko biloba' = 'Ginkgo biloba',
	'Ginko biloba' = 'Ginkgo biloba',
	# 'Ginko; Biloba' = 'Ginkgo biloba',
	'Ginko biloba princeton sentry' = 'Ginkgo biloba',
	'Gymnocladus dioica' = 'Gymnocladus dioicus',
	'Ilex x aquipernyi' = 'Ilex',
	'Ilex x meserveae' = 'Ilex',
	'Maagolia x soulangiana' = 'Magnolia x soulangiana',
	'Malus x hybrida' = 'Malus',
	'Malus x moerlandsii' = 'Malus',
	'Myrica pennsylvanica' = 'Myrica pensylvanica',
	'Oxydendron arboreum' = 'Oxydendrum arboreum',
	'Oxydendron spp' = 'Oxydendrum',
	'Phanus frangula' = 'Frangula alnus',
	'Picea glauca conica' = 'Picea glauca',
	'Picea omorica' = 'Picea omorika',
	'Prunus cistena' = 'Prunus x cistena',
	'Prunus x cistena' = 'Prunus x cistena',
	'Prunus pennsylvanica' = 'Prunus pensylvanica',
	'Prunus sargenti' = 'Prunus sargentii',
	'Prunus serrulata pendula' = 'Prunus serrulata',
	'Prunus serrulata ssp pubescens' = 'Prunus serrulata',
	'Prunus subhirtella pendula' = 'Prunus subhirtella',
	'Prunus x hybrida' = 'Prunus',
	'Prunus x incam' = 'Prunus',
	'Prunus x incam okame' = 'Prunus',
	'Quercus rubrum' = 'Quercus rubra',
	'Quercus x hybrida' = 'Quercus',
	'Qurecus rubra' = 'Quercus rubra',
	'Sciadopity verticillata' = 'Sciadopitys verticillata',
	'Sequoiadendron gigantea' = 'Sequoiadendron giganteum',
	'Spiraea x hybrida' = 'Spiraea',
	'Stewartia pseudocamillia' = 'Stewartia pseudocamellia',
	'Tilia x mongolica' = 'Tilia mongolica',
	'Ulmus thomasi' = 'Ulmus thomasii',
	'X cupressocyparis leylandii' = 'Cupressocyparis leylandii'
)
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})
# Should throw a list of NULLs

# # [non-correct move to reload taxa_map from file]
taxa_map <- trim_taxa(path = map_path)
### Do hack thing here -- first pass
# Make modified taxa_map2 df forcing ITIS misses into $taxa_clean 
#  => *Does Not Match File*
taxa_map2 <- taxa_map %>% 
	mutate(taxa_clean = ifelse(is.na(taxa_replacement), 
															taxa_clean, taxa_replacement))
# => Hackish: overwrite taxa_map.csv with taxa_map2
write_csv(taxa_map2, paste0(map_path, '/taxa_map.csv'))
#  Get changed taxa
changes <- taxa_map2[(! taxa_map2$taxa_raw %in% taxa_map2$taxa_clean)
										& (is.na(taxa_map2$taxa_trimmed)), c(1:3,5)]
# Get Eco PLANTS codes
spLsFile <- '../EcoSpeciesList_20July2018.csv'
SpecList <- read_csv(spLsFile, skip=1,
								 col_names=c('ID','Code','SName','CName','Genus','Host'))
eco_matches <- left_join(taxa_map2, SpecList,
											by=c('taxa_clean' = 'SName'))
#  Check for records that didn't find codes
length(table(eco_matches[is.na(eco_matches$Code),]$taxa_raw))
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)
	# Unmatched on Eco list
	#   Some of these are identities, to reset names that get changed
	#   by TCR authorities
repl_list <- list(
	'Abies balsamea var phanerolepis' = 'Abies x phanerolepis',
	# 'Abies koreana' = 'Abies koreana',
	'Acer freemanii' = 'Acer x freemanii',
	'Acer x freemanii' = 'Acer x freemanii',
	# 'Acer platanoides schwedleri' = 'Acer platanoides',
	# 'Acer rubrum columnare' = 'Acer rubrum',
	'Acer tataricum ginnala' = 'Acer tataricum',
	'Acer tataricum ssp ginnala' = 'Acer tataricum',
	'Aesculus octandra' = 'Aesculus flava',
	'Aesculus x arnoldiana' = 'Aesculus',
	'Aesculus x carnea' = 'Aesculus x carnea',
	'Alnus rugosa' = 'Alnus incana',
	'Amelanchier grandifolia' = 'Amelanchier x grandiflora',
	'Amelanchier x grandiflora' = 'Amelanchier x grandiflora',
	'Amelanchier x grandiflora [autumn brilliance]' = 'Amelanchier x grandiflora',
	'Betula szechuanica' = 'Betula platyphylla',
	'Buxus microphyla' = 'Buxus microphylla koreana',
	'Callitropsis nootkatensis' = 'Chamaecyparis nootkatensis',
	'Campsis radicans' = 'Campsis radicans', # not in Eco
	'Carya tomentosa' = 'Carya alba',
	# 'Cercis reniformis' = 'Cercis reniformis',
	'Cladrastis lutea' = 'Cladrastis kentukea',
	'Cladrastus lutea' = 'Cladrastis kentukea',
	# 'Crataegus crusgalli var. Inermis' = 'Crataegus crus-galli',
	'Crataegus crusgalli inermis' = 'Crataegus crus-galli',
	# 'Crataegus crusgalli var' = 'Crataegus crus-galli',
	# 'Crataegus crusgalli var inermis' = 'Crataegus crus-galli',
	'Crataegus oxyacantha' = 'Crataegus',
	'Crataegus x lavallei' = 'Crataegus x lavallei',
	'Enkianthus campanulatus' = 'Enkianthus campanulatus', # not in Eco
	# 'Evodia danielli' = 'Tetradium daniellii',
	# 'Fagus grandiflora' = 'Fagus grandifolia',
	# 'Fagus silvatica' = 'Fagus sylvatica',
	# 'Fagus sylvatica tricolor' = 'Fagus sylvatica',
	'Forsythia x intermedia' = 'Forsythia x intermedia', # Web returns 'Forsythia X intermedia'
	'Fraxinus lanceolata' = 'Fraxinus pennsylvanica',
	# 'Ginko biloba' = 'Ginkgo biloba',
	'Gleditsia triacanthos forma inermis' = 'Gleditsia triacanthos',
	'Gleditsia triacanthos inermis' = 'Gleditsia triacanthos',
	# 'Hamamelis mollis' = 'Hamamelis mollis',
	'Heptacodium miconioides' = 'Heptacodium miconioides',
	'Ilex pedunculosa' = 'Ilex',
	'Kalopanax pictus' = 'Kalopanax septemlobus', # ITIS returns pictus, but web says not accepted
	'Laburnum watereri' = 'Laburnum x watereri',
	'Laburnum x watereri' = 'Laburnum x watereri',
	'Laburnum_x_watereri' = 'Laburnum x watereri',
	'Magnolia soulangiana' = 'Magnolia x soulangiana',
	'Magnolia x soulangeana' = 'Magnolia x soulangiana',
	'Magnolia x soulangiana' = 'Magnolia x soulangiana',
	'Myrica heterophylla' = 'Myrica',
	'Myrica pennsylvanica' = 'Myrica',
	# 'Oxydendron spp' = 'Oxydendrum',
	'Paxistima canbyii' = 'Paxistima', # also should be 'canbyi'
	# 'Phanus frangula' = 'Frangula alnus',
	'Picea nigra' = 'Picea mariana', # ITIS returns nigra; web says not accepted
	# 'Picea omorica' = 'Picea omorika',
	'Picea pungens glauca' = 'Picea pungens',
	'Picea pungens glauca group' = 'Picea pungens',
	'Plantanus acerifolia' = 'Platanus x acerifolia',
	'Plantanus x acerifolia' = 'Platanus x acerifolia',
	'Platanus acerifoli' = 'Platanus x acerifolia',
	'Platanus acerifolia' = 'Platanus x acerifolia',
	'Platanus x acerfolia' = 'Platanus x acerifolia',
	'Platanus x acerifolia' = 'Platanus x acerifolia',
	'Potentilla fruiticosa' = 'Potentilla', # Eco has typo
	# 'Prunus pissardii' = 'Prunus pissardii', # ITIS returns nothing, web gives P. cerasifera
	# 'Prunus sargenti' = 'Prunus sargentii',
	# 'Prunus subhirtella pendula' = 'Prunus subhirtella',
	# 'Prunus x cistena' = 'Prunus x cistena',
	# 'Prunus x incam okame' = 'Prunus',
	'Quercus nuttallii' = 'Quercus texana',
	'Quercus robur fastigiata' = 'Quercus robur',
	'Quercus robur forma fastigiata' = 'Quercus robur',
	'Quercus virginiana' = 'Quercus/live virginiana',
	'Quercus x bebbiana' = 'Quercus x bebbiana',
	'Quercus x sargentii' = 'Quercus x sargentii',
	'Quercus x saulii' = 'Quercus x saulii',
	'Quercus x schuettei' = 'Quercus x schuettei',
	'Rhamnus frangula' = 'Frangula alnus',
	'Rhus typhina' = 'Rhus hirta',
	'Rosa hugonis' = 'Rosa',
	'Salix arenaria' = 'Salix',
	'Salix x cottetii' = 'Salix',
	'Salix x grahamii' = 'Salix',
	# 'Salix x sepulcralis simonkai' = 'Salix x sepulcralis Simonkai',
	'Spirea x bumalda' = 'Spiraea x bumalda',
	'Styphnolobium japonicum' = 'Sophora japonica',
	'Styphnoloboum japonicum' = 'Sophora japonica',
	# 'Styphnolobium japonica' = 'Sophora japonica',
	'Swida alternifolia' = 'Cornus alternifolia',
	'Syringa meyeri palibin' = 'Syringa',
	'Syringa pekinensis' = 'Syringa reticulata',
	'Syringa persica' = 'Syringa x persica',
	'Thuja orientalis' = 'Platycladus orientalis',
	# 'Tilia mongolica' = 'Tilia mongolica',
	'Tilia x europa' = 'Tilia x europaea',
	'Tilia x europaea' = 'Tilia x europaea',
	'Tilia x flavescens' = 'Tilia',
	'Ulmus capinifolia' = 'Ulmus',
	'Ulmus carpinifolia' = 'Ulmus',
	'Ulmus davidiana var japonica' = 'Ulmus davidiana',
	'Ulmus x hollandica' = 'Ulmus x hollandica',
	'Unknown cladrastis' = 'Cladrastis',
	'Viburnum sargentii' = 'Viburnum',
	'Viburnum trilobum' = 'Viburnum'
)
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})
# Should throw a list of NULLs
# [non-correct move to reload taxa_map from file]
taxa_map <- trim_taxa(path = map_path)
# Make modified taxa_map2 df forcing Eco names into $taxa_clean 
#  => *Does Not Match File*
taxa_map2 <- taxa_map %>% 
	mutate(taxa_clean = if_else(is.na(taxa_replacement), 
															taxa_clean, taxa_replacement))
#  Check again for missing codes (against taxa_map2) -- should be none
eco_matches <- left_join(taxa_map2, SpecList,
											by=c('taxa_clean' = 'SName'))
length(table(eco_matches[is.na(eco_matches$Code),]$taxa_raw))
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)
#  Get changed taxa; write csv of changed names
changes <- taxa_map2[(! taxa_map2$taxa_raw %in% taxa_map2$taxa_clean)
										& (is.na(taxa_map2$taxa_trimmed)), c(1:3,5)]
write_csv(changes, paste0(map_path, '/taxa_changed.csv'))
# => Hackish: overwrite taxa_map.csv with taxa_map2
write_csv(taxa_map2, paste0(map_path, '/taxa_map.csv'))
#  Revise the raw data table (from rewritten taxa_map.csv)
#  and write to file
taxa_output <- revise_taxa(path = map_path, x = commsDict, 
													 col = 'SciName', sep = '\t')
## Done with taxonomyCleanR


# Reduce to what we need -- AND TRIM CURSED WHITE SPACE
commsDict <- as_tibble(taxa_output[1:3]) %>% 
	mutate(taxa_clean = trimws(taxa_clean))
table(commsDict$taxa_clean[str_detect(commsDict$taxa_clean, 'Acer')])
# Stick on PLANTS codes
commsDict <- left_join(commsDict, SpecList,
											by=c('taxa_clean' = 'SName')) 
# Check for code failures (a lot, but almost all non-taxa)
# table(commsDict$taxa_clean, useNA='always')
# table(commsDict$SciName[is.na(commsDict$Code)])
table(commsDict$taxa_clean[is.na(commsDict$Code)])

# Eliminate entries for SciName typos &c.
commsDictUn <- unique(commsDict[2:3]) # n=2040

# The hope is for each CommName to point to one SciName;
# this collects ones like Apple that point to several.
ununiqs <- commsDictUn %>% 
	group_by(CommName) %>% filter(n() > 1) %>%
	arrange(CommName) # n=264
# This makes a file of non-uniques that need judgment calls
#  for manipulating into the badMatches list that follows
write.csv(ununiqs, paste0(out_path, '/determinenda.csv'), row.names=FALSE)

# Manually composed list of extra entries to remove from commsDict
# => commented-out means NOT bad so we use it <=
badMatches <- tribble( 
	~CommName,	~taxa_clean,
	# 'american elm',	'Ulmus americana',
	'american elm',	'Ulmus',
	# 'american hornbeam',	'Carpinus caroliniana',
	'american hornbeam',	'Carpinus',
	'amur maackia',	'Acer pseudoplatanus',
	# 'amur maackia',	'Maackia amurensis',
	# 'amur maple',	'Acer ginnala',
	'amur maple',	'Acer tataricum',
	# 'apple',	'Malus',
	'apple',	'Malus domestica',
	'apple',	'Malus pumila',
	'arborvitae',	'Thuja',
	# 'arborvitae',	'Thuja occidentalis',
	# 'ash',	'Fraxinus',
	'ash',	'Fraxinus nigra',
	'ash',	'Fraxinus pennsylvanica',
	# 'autumn brilliance serviceberry',	'Amelanchier',
	'autumn brilliance serviceberry',	'Amelanchier x grandiflora',
	# 'basswood',	'Tilia',
	'basswood',	'Tilia americana',
	'beech copper',	'Fagus',
	# 'beech copper',	'Fagus sylvatica',
	# 'birch',	'Betula',
	'birch',	'Betula lenta',
	'black locust',	'Na na',
	# 'black locust',	'Robinia pseudoacacia',
	'border forsythia',	'Forsythia',
	# 'border forsythia',	'Forsythia x intermedia',
	# 'boxelder',	'Acer negundo',
	'boxelder',	'Acer nigrum',
	'bradford',	'Planting site small',
	'bradford',	'Planting site upright',
	'buckthorn',	'Frangula alnus',
	# 'buckthorn',	'Rhamnus',
	'buckthorn',	'Rhamnus cathartica',
	# 'carolina silverbell',	'Halesia carolina',
	'carolina silverbell',	'Halesia tetraptera',
	# 'catalpa',	'Catalpa',
	'catalpa',	'Catalpa erubescens',
	'catalpa',	'Catalpa speciosa',
	'cherry',	'Planting site small',
	# 'cherry',	'Prunus',
	'cherry',	'Prunus subhirtella',
	# 'cherry accolade',	'Prunus',
	'cherry accolade',	'Prunus sargentii',
	# 'cherry kwanzan',	'Prunus',
	'cherry kwanzan',	'Prunus serrulata',
	'common apple',	'Malus pumila',
	# 'common apple',	'Malus',
	'common apple',	'Malus sylvestris',
	# 'cottonwood',	'Populus',
	'cottonwood',	'Populus deltoides',
	'crabapple',	'Ma',
	# 'crabapple',	'Malus',
	# 'crabapple species',	'Malus',
	'crabapple species',	'Planting site medium',
	'crimson spire oak',	'Quercus alba',
	# 'crimson spire oak',	'Quercus robur',
	# 'dogwood',	'Cornus',
	'dogwood',	'Cornus florida',
	'doublefile viburnum',	'Viburnum',
	# 'doublefile viburnum',	'Viburnum plicatum',
	# 'eastern redbud',	'Cercis canadensis',
	'eastern redbud',	'Cercis',
	'eastern white cedar',	'Juniperus virginiana',
	# 'eastern white cedar',	'Thuja occidentalis',
	# 'elm',	'Ulmus',
	'elm',	'Ulmus americana',
	# 'elm accolade',	'Ulmus',
	'elm accolade',	'Ulmus japonica',
	# 'elm triumph',	'Ulmus',
	'elm triumph',	'Ulmus japonica',
	'english elm',	'Ulmus minor',
	# 'english elm',	'Ulmus procera',
	# 'english hawthorn',	'Crataegus laevigata',
	'english hawthorn',	'Crataegus',
	# 'european linden',	'Tilia',
	'european linden',	'Tilia x europaea',
	# 'flowering cherry',	'Prunus',
	'flowering cherry',	'Prunus serrulata',
	# 'flowering crabapple',	'Malus',
	'flowering crabapple',	'Malus spectabilis',
	# 'flowering dogwood',	'Cornus florida',
	'flowering dogwood',	'Cornus kousa',
	# 'ginkgo',	'Ginkgo biloba',
	'ginkgo',	'Planting site upright',
	'gold mop japanese falsecypress',	'Chamaecyparis obtusa',
	# 'gold mop japanese falsecypress',	'Chamaecyparis pisifera',
	'goldenrain tree',	'Koelreuteria elegans',
	# 'goldenrain tree',	'Koelreuteria paniculata',
	# 'hawthorne',	'Crataegus',
	'hawthorne',	'Crataegus phaenopyrum',
	# 'hedge maple',	'Acer campestre',
	'hedge maple',	'Planting site small',
	# 'hemlock',	'Tsuga',
	'hemlock',	'Tsuga canadensis',
	# 'holly',	'Ilex',
	'holly',	'Ilex opaca',
	# 'honeylocust skyline',	'Gleditsia',
	'honeylocust skyline',	'Gleditsia triacanthos',
	# 'honeylocust thornless',	'Gleditsia',
	'honeylocust thornless',	'Gleditsia triacanthos',
	# 'hornbeam',	'Carpinus',
	'hornbeam',	'Ostrya virginiana',
	# 'hornbeam-american',	'Carpinus caroliniana',
	'hornbeam-american',	'Ostrya virginiana',
	# 'horse chestnut',	'Aesculus hippocastanum',
	'horse chestnut',	'Castanea dentata',
	# 'horsechestnut',	'Aesculus',
	'horsechestnut',	'Aesculus hippocastanum',
	'ironwood',	'Carpinus betulus',
	# 'ironwood',	'Carpinus caroliniana',
	'japanese falsecypress',	'Chamaecyparis obtusa',
	# 'japanese falsecypress',	'Chamaecyparis pisifera',
	# 'japanese tree lilac',	'Syringa reticulata',
	'japanese tree lilac',	'Syringa vulgaris',
	'lacebark elm',	'Ulmus',
	# 'lacebark elm',	'Ulmus parvifolia',
	# 'leonard messel magnolia',	'Magnolia',
	'leonard messel magnolia',	'Magnolia kobus',
	# 'leyland cypress',	'Cupressocyparis leylandii',
	'leyland cypress',	'Cupressus',
	# 'lilac',	'Syringa',
	'lilac',	'Syringa vulgaris',
	# 'lilac japanese tree',	'Syringa reticulata',
	'lilac japanese tree',	'Syringa vulgaris',
	# 'limber pine',	'Pinus flexilis',
	'limber pine',	'Pinus',
	'linden',	'Planting site medium',
	'linden',	'Planting site small',
	'linden',	'Planting ste medium',
	# 'linden',	'Tilia',
	'linden',	'Tilia americana',
	'linden',	'Tilia euchlora',
	'linden',	'Tilia cordata',
	# 'linden glenleven',	'Tilia',
	'linden glenleven',	'Tilia cordata',
	'loebner magnolia',	'Magnolia stellata',
	# 'loebner magnolia',	'Magnolia',
	# 'london planetree',	'Platanus x acerifolia',
	'london planetree',	'Platanus hybrida',
	'london planetree',	'Platanus occidentalis',
	# 'magnolia saucer',	'Magnolia x soulangiana',
	'magnolia saucer',	'Magnolia',
	# 'maple',	'Acer',
	'maple',	'Acer truncatum',
	# 'maple amur',	'Acer ginnala',
	'maple amur',	'Acer tataricum',
	# 'maple norway',	'Acer platanoides',
	'maple norway',	'Picea',
	# 'maple silver',	'Acer saccharinum',
	'maple silver',	'Malus',
	'maple sugar',	'Acer saccharinum',
	# 'maple sugar',	'Acer saccharum',
	'mimosa',	'Albizia julibrissin',
	'mimosa',	'Albizia',
	'mimosa',	'Mimosa',
	# 'mulberry',	'Morus',
	'mulberry',	'Morus alba',
	'no planting site',	'No planting site',
	'no planting site',	'No plantng site',
	'norway',	'Planting site medium',
	'norway',	'Planting site upright',
	# 'norway spruce',	'Picea abies',
	'norway spruce',	'Pinus strobus',
	# 'oak',	'Quercus',
	'oak',	'Quercus alba',
	'other - see comments',	'Cercis',
	'other - see comments',	'Na na',
	# 'peach',	'Prunus persica',
	'peach',	'Prunus',
	'pear',	'Na na',
	# 'pear',	'Pyrus',
	'pear',	'Pyrus calleryana',
	'pear',	'Pyrus communis',
	# 'persimmon',	'Diospyros',
	'persimmon',	'Diospyros virginiana',
	# 'pin cherry',	'Prunus pensylvanica',
	'pin cherry',	'Prunus',
	'pin oak',	'Acer platanoides',
	# 'pin oak',	'Quercus palustris',
	'planting site',	'Planting note',
	'planting site',	'Planting site',
	'planting site',	'Planting site - hold',
	'red oak',	'Quercus falcata',
	# 'red oak',	'Quercus rubra',
	'regal prince oak',	'Quercus robur',
	# 'regal prince oak',	'Quercus',
	'russian olive',	'Elaeagnus angustifolia',
	# 'russian olive',	'Elaeagnus',
	# 'sargent crabapple',	'Malus sargentii',
	'sargent crabapple',	'Malus',
	# 'sargent viburnum',	'Viburnum',
	'sargent viburnum',	'Viburnum sieboldii',
	'serbian spruce',	'Picea abies',
	# 'serbian spruce',	'Picea omorika',
	# 'serviceberry',	'Amelanchier',
	'serviceberry',	'Amelanchier canadensis',
	'serviceberry',	'Amelanchier x grandiflora',
	'serviceberry',	'Amelanchier laevis',
	'serviceberry autumn brilliance',	'Amelanchier laevis',
	# 'serviceberry autumn brilliance',	'Amelanchier x grandiflora',
	'serviceberry robin hill',	'Amelanchier',
	# 'serviceberry robin hill',	'Amelanchier x grandiflora',
	'shadblow',	'Amelanchier canadensis',
	'shadblow',	'Amelanchier laevis',
	# 'siebold viburnum',	'Viburnum sieboldii',
	'siebold viburnum',	'Viburnum',
	'silver maple',	'Acer rubrum',
	# 'silver maple',	'Acer saccharinum',
	'silver maple',	'Planting site small',
	# 'slippery elm',	'Ulmus rubra',
	'slippery elm',	'Ulmus',
	'smoke tree',	'Cotinus coggygria',
	# 'smoke tree',	'Cotinus',
	'sourwood',	'Oxydendrum arboreum',
	# 'sourwood',	'Oxydendrum',
	# 'spruce',	'Picea',
	'spruce',	'Picea abies',
	# 'spruce norway',	'Picea abies',
	'spruce norway',	'Picea',
	'stella hybrid dogwood',	'Cornus florida',
	'stella hybrid dogwood',	'Unresolved',
	'sugar maple',	'Acer saccharinum',
	# 'sugar maple',	'Acer saccharum',
	'sugarberry',	'Celtis laevigata',
	# 'sugarberry',	'Celtis',
	# 'sweetgum',	'Liquidambar styraciflua',
	'sweetgum',	'Some roots',
	'tamarack',	'Larix',
	# 'tamarack',	'Larix laricina',
	# 'tree lilac',	'Syringa',
	'tree lilac',	'Syringa reticulata',
	# 'tree of heaven',	'Ailanthus',
	'tree of heaven',	'Ailanthus altissima',
	'turkey oak',	'Quercus cerris',
	'turkey oak',	'Quercus laevis',
	'undefined',	'A plant site no wires',
	'undefined',	'A plant site wires',
	'unknown',	'Carya',
	'unknown',	'Other',
	'unknown',	'Unidentified',
	'unknown',	'Unknown spp',
	'unknown',	'Unknown unknown',
	# 'upright sweetgum',	'Liquidambar',
	'upright sweetgum',	'Liquidambar styraciflua',
	# 'viburnum',	'Viburnum',
	'viburnum',	'Viburnum sieboldii',
	# 'weeping cherry',	'Prunus',
	'weeping cherry',	'Prunus serrulata',
	'weeping cherry',	'Prunus subhirtella',
	# 'weeping redbud',	'Cercis',
	'weeping redbud',	'Cercis canadensis',
	'weeping willow',	'Salix alba',
	'weeping willow',	'Salix babylonica',
	'weeping willow',	'Salix matsudana',
	# 'witch-hazel',	'Hamamelis',
	'witch-hazel',	'Hamamelis virginiana',
	# 'witch hazel',	'Hamamelis',
	'witch hazel',	'Hamamelis virginiana',
	'yellow wood',	'Cladrastis kentukea',
	'yellow wood',	'Unresolved',
	# 'zelkova',	'Zelkova',
	'zelkova',	'Zelkova serrata'
) # n=157

# Take out undesired entries (into new df in case you want to go back)
commsDict2 <- commsDictUn %>% 
	anti_join(badMatches) %>%
	transmute(CommName, SciName = taxa_clean) # n=1883
length(table(commsDict2$SciName)) 	# n=524

# Assuming all's well,
commsDict <- commsDict2

# Additional additions
# NY_Geneva
addtn <- tribble(
	~CommName,               ~SciName,
	'Eastern Service Berry', 'Amelanchier canadensis',
	'Smoothleaf Elm', 'Ulmus minor'
)
commsDict <- bind_rows(commsDict, addtn)

# NY_Rochester
addtn <- tribble(
	~CommName,               ~SciName,
	'Arnold hawthorn', 'Crataegus',
	'Chinese magnolia; Saucer', 'Magnolia x soulangeana',
	'Common persimmon', 'Diospyros',
	'Jack pine', 'Pinus banksiana',
	'Spruce pine', 'Pinus'
)
commsDict <- bind_rows(commsDict, addtn)

# NH_Manchester
addtn <- tribble(
	~CommName,               ~SciName,
	'European birch', 'Betula pendula',
	'Hydrangea', 'Hydrangea',
	'Mountain-ash', 'Sorbus',
	'Ornamental maple', 'Acer',
	'Red/Black oak', 'Quercus',
	'Russian mulberry', 'Morus',
	'Trembling aspen', 'Populus tremuloides'
)
commsDict <- bind_rows(commsDict, addtn)

# MA_Somerville
addtn <- tribble(
	~CommName,               ~SciName,
	'cherry/plum', 'Prunus',
	'common alder', 'Alnus',
	'European mountainash', 'Sorbus aucuparia',
	'thornless honeylocust', 'Gleditsia triacanthos'
)
commsDict <- bind_rows(commsDict, addtn)

# MA_Somerville_street
addtn <- tribble(
	~CommName,               ~SciName,
	'oak northern pin', 'Quercus ellipsoidalis',
	'northern pin oak', 'Quercus ellipsoidalis'
)
commsDict <- bind_rows(commsDict, addtn)


# Writing most final version to file 
write_csv(commsDict, 'commNamesDictionary/commonsDict.csv')

