##Written by Emma Tait for FEMC Urban Pests Project
## this script connects to MS Access and opens up iTree .ieco files
## it caclutes values for carbon, pollution, runoff and structure
## it returns a csv file with these values and the location.
## this script can be used with a single file or with a list of files
## Last Updated: ERT 18 May 2020

import pyodbc
import csv
import datetime


#the input csv with a column containing a list of filenames of the files that you want to process. These files must all be in the same location
infile = r'I:\WorkingProjects\EconomicImpacts_UrbanPests\FinalDataProcessing\Inputs\iEcoFileList.csv'
##infile = r'E:\Emma\Documents\FEMC\urbanPests\testingAccessScriptList.csv'
#column indexes for values in the infile
invCodeCol = 0
filenameCol = 1
projectIdCol = 2
datasetIdCol = 3
yearsCol = 4
placeCol = 5
##ecofilepath= r'I:\WorkingProjects\EconomicImpacts_UrbanPests\Data_for_iTree\iTreeECO_files'
ecofilepath = r'I:\WorkingProjects\EconomicImpacts_UrbanPests\Data_for_iTree\iTreeECO_files'
pestFile = r'I:\WorkingProjects\EconomicImpacts_UrbanPests\FinalDataProcessing\Inputs\PestsHosts.csv'
generaLookup = r'I:\WorkingProjects\EconomicImpacts_UrbanPests\FinalDataProcessing\Inputs\generaLookup.csv'
# If you want to add records to an existing csv change this to True
append = False
#file name and path for the csv with state county and municipality code lookup
codefile = r'I:\WorkingProjects\EconomicImpacts_UrbanPests\FinalDataProcessing\Inputs\finalMatchedPlaces.csv'
#file name and path for final csv. If 'append' is True then it will add to the this file or make the file if it does not exist.
#if append is false then it will create or overwrite the file here
pestOutfile = r'I:\WorkingProjects\EconomicImpacts_UrbanPests\FinalDataProcessing\Outputs\pestTreeValues.csv'
invOutfile = r'I:\WorkingProjects\EconomicImpacts_UrbanPests\FinalDataProcessing\Outputs\inventoryMetrics.csv'

#main functions. Opens output csv and gets values for each file in the input file list
def main ():
    pests = importPestList(pestFile)
    ecofiles = makeFileList(infile)
    
    method = ''
    if append:
        method = 'a'
    else:
        method = 'w'
    #for each ecofile in the list get the pest data and the inventory data
    
        #create the pest list output file
    with open (pestOutfile,method,newline='') as outputcsv:
        pestwriter = csv.writer(outputcsv, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
        pestfieldnames=['inventory_code','pest_shortname','pest','metric','value']
        if not append:
            pestwriter.writerow(pestfieldnames)
        #write the pest values to the file
        
        #create the inventory output file    
        with open (invOutfile,method,newline='') as outputcsv:
            ecowriter = csv.writer(outputcsv, delimiter=',',quotechar='|', quoting=csv.QUOTE_MINIMAL)
            fieldnames=['inventory_code','category','metric','SppScientificName','SppCommonName','value']
            if not append:
                ecowriter.writerow(fieldnames)
            for f in ecofiles:
                print ('\n'+str(datetime.datetime.now()))
                print ('Processing file #'+str(ecofiles.index(f)+1)+': '+f['file'])
                getPestValues(f['file'],pestwriter,pests)
                print ('pest csv processed')  
                ecowriter.writerow([f['file'][:-5],'inventory','projectID','','',f['projectID']])
                #write the inventory info data to the file
                getInventoryInfo(f['file'],ecowriter,f['startYR'],f['endYR'],f['place'])
                print ('inventory metrics processed')
    print ('\npest csv written to: '+pestOutfile)
    print ('inventory metrics written to'+invOutfile)
    print ('--script complete--')
    

#imports a CSV of pest names and host species
#returns a dictionary of pests and their host species and genus
def importPestList(pestfile):
    pests = {}
    with open(pestfile, newline='')as pestcsv:
        pestlist = csv.reader(pestcsv,delimiter=',')
        #skips header row
        next(pestlist)
        for pest in pestlist:
            if pests.get(pest[2]+'_'+pest[5]) == None:
                pests[pest[2]+'_'+pest[5]] = ''
            
            if ' spp' in pest[1]:
                pests[pest[2]+'_'+pest[5]]+=" OR SppScientificName LIKE '"+pest[1][0:pest[1].index(' spp')]+"%'"
            else:
                pests[pest[2]+'_'+pest[5]]+=" OR SppScientificName LIKE '"+pest[1]+"%'"
    return pests
    
#processes the infile to determine if it is a single file or a csv with a list of files.
#if it is a csv, it gets the filenames from the indicated column
#Returns a list
def makeFileList (infile):
    print ('making file list')
    ecofiles = []
##    if infile[-4:len(infile)] == 'ieco':
##        ecofile.append(infile)
##    else:
    print(infile)
    with open(infile, newline='') as csvfile:         
         filereader = csv.reader(csvfile, delimiter=',')
         #skips header row
         next(filereader)
         for row in filereader:
             eco = dict()
             eco['file'] = row[filenameCol]
             eco['projectID'] = row[projectIdCol]
             eco['startYR'] = int(row[yearsCol].split('-')[0]) if row[yearsCol] != '' else 0
             eco['endYR'] = int(row[yearsCol].split('-')[1]) if row[yearsCol] != '' else 0
             eco['place'] = row[placeCol]
##             if row[datasetIdCol] != '':
##                 eco['datasetID'] = row[datasetIdCol]
             ecofiles.append(eco)
    return ecofiles

#gets inventory information from MS access and writes it to the inventory csv
def getInventoryInfo(ecofile,writer,startyr, endyr, place):
    #array to hold names of top 10 species for inventory
    top10spp = []
    topGenera = {}
##    topSpp = {}
##    invlist = {}
##    invlist['tree'] = {}
##    invlist['inventory']={}
##    invlist['genera'] = {}
##    
    #MS Access database connection
    conn = pyodbc.connect(r'DRIVER={Microsoft Access Driver (*.mdb, *.accdb)};DBQ='+ecofilepath+'\\'+ecofile+';')
    cursor = conn.cursor()
    #gets basic inventory info: place codes, population, year, date inventory was run through iTree
    print ('getting inventory information')
    invInfo = '''
     SELECT
            (SELECT PrimaryPartitionCode FROM EcoProjectLocations) as primary,
            (SELECT SecondaryPartitionCode FROM EcoProjectLocations) as secondary, 
            (SELECT TertiaryPartitionCode FROM EcoProjectLocations) as tertiary,
            (SELECT Population FROM EcoYearLocationData) as population,
            (SELECT  y.Year FROM (EcoProjects AS pr INNER JOIN EcoSeries AS s ON pr.ProjectKey = s.ProjectKey) INNER JOIN EcoYears AS y ON s.SeriesKey = y.SeriesKey;) as invYear,
            DateValue(DateTime) as runDate
        FROM EcoYearResults
        '''
    #writes basic info to file
    result = cursor.execute(invInfo).fetchall()
    if len(result) == 1:
        row = result[0]
    else:
        row = result[len(result) - 1]
    startyr = startyr if startyr != 0 else row.invYear
    endyr = endyr if endyr != 0 else row.invYear
    municipalityText = getMunicipalityText (row.primary,row.secondary,row.tertiary)
    place = place if municipalityText[3] == '' else municipalityText[3]
    writer.writerow([ecofile[:-5],'inventory','iTree_run_date','','',row.runDate.strftime("%m/%d/%Y")])
    writer.writerow([ecofile[:-5],'inventory','inventory_start_year','','',startyr])
    writer.writerow([ecofile[:-5],'inventory','inventory_end_year','','',endyr])
    writer.writerow([ecofile[:-5],'inventory','population','','',row.population])
    writer.writerow([ecofile[:-5],'inventory','state','','',municipalityText[0]])
    writer.writerow([ecofile[:-5],'inventory','county','','',municipalityText[1]])
    writer.writerow([ecofile[:-5],'inventory','municipality','','',municipalityText[2]])
    writer.writerow([ecofile[:-5],'inventory','municipality_1','','',place])
    writer.writerow([ecofile[:-5],'inventory','state_code','','',row.primary])
    writer.writerow([ecofile[:-5],'inventory','county_code','','',row.secondary])
    writer.writerow([ecofile[:-5],'inventory','municipality_code','','',row.tertiary])

    #get total number of trees recorded in inventory

    total_trees = '''
        SELECT COUNT(TreeId) as total_tree_count from IndividualTreeEffects
    '''
    result = cursor.execute(total_trees).fetchall()
    row = result[0]
    writer.writerow([ecofile[:-5],'inventory','total_tree_count','','',row.total_tree_count])
    
    #gets metrics for top 10 most inventoried species: percent population, percent leaf area, importance value
    print ('calculating species metrics')
    metrics = '''
    SELECT TOP 10
            SppScientificName,
            SppCommonName,
            SppTreeCount,
            ((SppTreeCount/(SELECT COUNT(TreeId) FROM IndividualTreeEffects))*100) as percent_population,
            ((SppLeafArea/(SELECT SUM(LeafArea) FROM IndividualTreeEffects))*100) as percent_leaf_area,
            (percent_population+percent_leaf_area) as importance_value
        FROM
            (SELECT
                SppScientificName,
                SppCommonName,
                COUNT(TreeId) as SppTreeCount,
                SUM(LeafArea) as SppLeafArea
            FROM
                IndividualTreeEffects
            GROUP BY sppScientificName, SppCommonName) as spp
        ORDER BY SppTreeCount DESC
    '''
    #writes metrics to the inventory csv
    for row in cursor.execute(metrics).fetchall():
        #make list of top ten most queried species for the next query
        writer.writerow([ecofile[:-5],'tree','percent_population',row.SppScientificName,row.SppCommonName,row.percent_population])
        writer.writerow([ecofile[:-5],'tree','percent_leaf_area',row.SppScientificName,row.SppCommonName,row.percent_leaf_area])
        writer.writerow([ecofile[:-5],'tree','importance_value',row.SppScientificName,row.SppCommonName,row.importance_value])

    
    print ('calculating size class')
    #gets number of trees of the top ten genera.
    topGenera = ''' SELECT 
         TOP 10
                  COUNT(DBH) as  numTrees, genera FROM(SELECT iif(InStr(sppScientificName,' ')=0, SppScientificName,Left(SppScientificName,InStr(SppScientificName,' ')-1)) as genera,DBH
            from IndividualTreeEffects)
            Group by genera
            ORDER BY  COUNT(DBH) DESC'''
    
              

    records = cursor.execute(topGenera).fetchall()
    for row in records:
        top10spp.append(row.genera)
        
        with open(generaLookup, newline='') as csvfile:         
         filereader = csv.reader(csvfile, delimiter=',')
         #skips header row
         next(filereader)
         for r in filereader:
             if r[0].lower() == (row.genera).lower():
                writer.writerow([ecofile[:-5],'genera','total_trees',row.genera,r[1],row.numTrees])

    #gets size class information for top ten most inventoried genera
##    print (top10spp)
    sizeclass = ''' SELECT 
              COUNT(sizeClass)  as count,  sizeClass,genera
                FROM (SELECT iif(InStr(sppScientificName,' ')=0, SppScientificName,Left(SppScientificName,InStr(SppScientificName,' ')-1)) as genera,
                            switch(
                                    DBH <= 10, '0-10',
                                    DBH <= 20,  '10-20',
                                    DBH <= 30,  '20-30',
                                    DBH <= 40, '30-40',
                                    DBH > 40, '>40'
                                    ) AS sizeClass
                from IndividualTreeEffects WHERE iif(InStr(sppScientificName,' ')=0, SppScientificName,Left(SppScientificName,InStr(SppScientificName,' ')-1)) IN ({}))
                Group by sizeClass,genera
                ORDER BY genera'''
    

    #puts placeholders in string for the variables in the list of top 10 species
    qmarks = ','.join('?' * len(top10spp))
    sizeclass = sizeclass.format(qmarks)
    #writes size class info to inventory file
    for row in cursor.execute(sizeclass,(top10spp)).fetchall():
        with open(generaLookup, newline='') as csvfile:         
         filereader = csv.reader(csvfile, delimiter=',')
         #skips header row
         next(filereader)
         for r in filereader:
             if r[0].lower() == (row.genera).lower():
                 writer.writerow([ecofile[:-5],'genera',row.sizeClass,r[0],r[1],row.count])

    
#connects to MS Access and queries the ieco file tables to return the desired values according to pest types. Then writes the values to the output csv
def getPestValues (ecofile,writer, pests):
    print('getting pest values')
    #for each pest, gets the value for pollution removal, avoided runoff, carbon sequestration and structural value for their host species
    for pest in pests:
        pest_short = pest.split('_')[1]
        pestname = pest.split('_')[0]
        conn = pyodbc.connect(r'DRIVER={Microsoft Access Driver (*.mdb, *.accdb)};DBQ='+ecofilepath+'\\'+ecofile+';')
        cursor = conn.cursor()
        full = '''
            SELECT
                SUM (structure) AS structural_value,
                SUM (carbon) AS carbon_sequestration,
                SUM (runoff) AS avoided_runoff,
                SUM (pollution) AS pollution_removal,
                SUM (sppCount) as tree_count,
                (SUM(carbon)+SUM(runoff)+SUM(pollution)) AS annual_value
            FROM (
                SELECT 
                    IndividualTreeEffects.SppScientificName,
                    IndividualTreeEffects.SppCommonName,
                    COUNT (TreeId) as sppCount,
                    SUM(IndividualTreeEffects.TreeValue) as structure, 
                    SUM((GrossCarbonSeq * (SELECT Price from EcoElementPrices WHERE Element = 'Carbon'))/1000) as carbon,
                    SUM(AvoidedRunoff * (SELECT Price from EcoElementPrices WHERE Element = 'InterceptedWater')) as runoff,
                    SUM(O3RemovalValue + NO2RemovalValue + PM25RemovalValue+ SO2RemovalValue) as pollution
                FROM (
                    SELECT
                        O3RemovalValue,
                        NO2RemovalValue,
                        PM25RemovalValue,
                        SO2RemovalValue,
                        IndividualTreeEffects.SppScientificName,
                        IndividualTreeEffects.SppCommonName,
                        IndividualTreeEffects.TreeValue,
                        IndividualTreeEffects.TreeId,
                        GrossCarbonSeq,
                        AvoidedRunoff 
                    FROM
                        IndividualTreeEffects
                    INNER JOIN
                        IndividualTreePollutionEffects
                    ON IndividualTreeEffects.TreeId = IndividualTreePollutionEffects.TreeId ) as fullTree
                    GROUP BY SppScientificName, SppCommonName) as spp
                WHERE '''+pests[pest][3:]
        cursor.execute(full)
        #writing pest info to csv
        for row in cursor.fetchall():
            writer.writerow([ecofile[:-5],pest_short,pestname,'carbon_sequestration',row.carbon_sequestration])
            writer.writerow([ecofile[:-5],pest_short,pestname,'avoided_runoff',row.avoided_runoff])
            writer.writerow([ecofile[:-5],pest_short,pestname,'pollution_removal',row.pollution_removal])
            writer.writerow([ecofile[:-5],pest_short,pestname,'structural_value',row.structural_value])
            writer.writerow([ecofile[:-5],pest_short,pestname,'annual_value',row.annual_value])
            writer.writerow([ecofile[:-5],pest_short,pestname,'tree_count',row.tree_count])
            
#gets the names of the state, county and town from the fips codes
def getMunicipalityText (statecode,countycode,towncode):
    with open(codefile, newline='')as codecsv:
        codelist = csv.reader(codecsv,delimiter=',')
        #skips header row
        next(codelist)
        state = ''
        county = ''
        town = ''
        place = ''
        statecode = statecode if statecode[0] != '0' else statecode.replace('0','')
        for code in codelist:
            if (code[0] == statecode and code[1] == countycode and (code[2]== towncode or code[3] == towncode)) or (code[0] == statecode  and (code[2]== towncode or code[3] == towncode)):
                state = code[5]
                county = code[6]
                town = code[7]
                if code[8] != '':
                    place = code[8]
                break
            elif code[0] == statecode and code[1] == countycode:
                state = code[5]
                county = code[6]
            elif code[0] == statecode:
                state = code[5]
    return [state,county,town,place]
    
main()
